package org.landis.jlyrics;

//{{{ imports
import javax.swing.*;
import javax.swing.filechooser.*;
import javax.swing.filechooser.songtxtFilter;
import java.awt.event.*;
import java.util.Vector;
import java.util.Iterator;
import java.util.Collections;
import java.io.File;
import org.landis.jlyrics.song.*;
import org.landis.jlyrics.*;
//}}}

public class MainWindow extends JFrame {
	//{{{ variables
	private javax.swing.JPanel pane = new JPanel();
	private java.util.Vector songIndex = new Vector();
	private java.util.Vector artistIndex = new Vector();
	private javax.swing.JList songJList;
	private org.landis.jlyrics.song.SongList songList = new SongList();
	private org.landis.jlyrics.jLyrics mainClass; //}}}
	
	//{{{ MainWindow()
	public MainWindow(jLyrics parent) {
		super("jLyrics");
		setBounds(250,150,400,400);
		mainClass = parent;
		makeData();
		init();
		setContentPane(pane);
		mainClass.setDirty(false);
	} //}}}
	
	//{{{ init()
	private void init() {
		/** setup menu
		 *
		 */
		javax.swing.JMenuBar menuBar = new JMenuBar();
		setJMenuBar(menuBar);
		javax.swing.JMenu menu;
		javax.swing.JMenuItem menuItem;
		
		menu = new JMenu("File");
		menuBar.add(menu);
			menuItem = new JMenuItem(new ImportAction(null, "Import", "Import a \".song.txt\" file"));
			menu.add(menuItem);
			menuItem = new JMenuItem(new ExportAction(null, "Export", "Export a Song to a \".song.txt\" file"));
			menu.add(menuItem);
			menuItem = new JMenuItem(new ExportXMLAction(null, "Export as XML", "Export the Songs as XML"));
			menu.add(menuItem);
			menu.addSeparator();
			menuItem = new JMenuItem(new SaveAction(null, "Save", "Saves the current List"));
			menu.add(menuItem);
			menuItem = new JMenuItem(new ExitAction("Exit", null));
			menu.add(menuItem);
		menu = new JMenu("Edit");
		menuBar.add(menu);
			menuItem = new JMenuItem(new AddAction(null, "Add Song", "Add New Song"));
			menu.add(menuItem);
			menuItem = new JMenuItem(new EditAction(null, "Edit Song", "Edit current song"));
			menu.add(menuItem);
			menuItem = new JMenuItem(new DeleteAction(null, "Delete", "Deletes the Selected Song"));
			menu.add(menuItem);
			menu.addSeparator();
			menuItem = new JMenuItem(new CopyAction(null, "Copy Song", "Copies the song to the clipboard"));
			menu.add(menuItem);
		menu = new JMenu("Options");
		menuBar.add(menu);
			menuItem = new JMenuItem(new OptionAction(null, "Options...", "Change Your Options"));
			menu.add(menuItem);
		/** setup toolbar
		 *
		 */
		javax.swing.JToolBar toolbar = new JToolBar();
		toolbar.setFloatable(false);
		javax.swing.JButton button = null;
		button = new JButton(new AddAction(new ImageIcon("./images/add.gif"), "", "Add New Song"));
		toolbar.add(button);
		button = new JButton(new EditAction(new ImageIcon("./images/edit.gif"), "", "Edit current song"));
		toolbar.add(button);
		button = new JButton(new DeleteAction(new ImageIcon("./images/delete.gif"), "", "Deletes the Selected Song"));
		toolbar.add(button);
		toolbar.addSeparator();
		button = new JButton(new ImportAction(new ImageIcon("./images/import.gif"), "", "Import a \".song.txt\" file"));
		toolbar.add(button);
		button = new JButton(new ExportAction(new ImageIcon("./images/export.gif"), "", "Export a song to a \".song.txt\" file"));
		toolbar.add(button);
		toolbar.addSeparator();
		button = new JButton(new CopyAction(new ImageIcon("./images/copy.gif"), "", "Copies the Selected Song to the Clipboard"));
		toolbar.add(button);
		/** setup list
		 *
		 */
		songJList = new JList() {
			public String getToolTipText(MouseEvent event) {
				int i = locationToIndex(event.getPoint());
				if (i < 0) return null;
				String newArtist = (String) artistIndex.get(i);
				if (newArtist.trim().equals(""))
					return "Artist: unknown";
				else
					return (String)"Artist: " + newArtist;
			}
		};
		updateData();
		songJList.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
		songJList.setVisibleRowCount(10);
		songJList.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 2) {
					editSong();
				}
			}
		});
		javax.swing.JScrollPane songListScroller = new JScrollPane(songJList);
		pane.add(toolbar);
		pane.setLayout(new BoxLayout(pane, BoxLayout.Y_AXIS));
		pane.add(songListScroller);
	} //}}}
	
	//{{{ MainWindow Methods
	
	//{{{ addSong()
	public void addSong(org.landis.jlyrics.song.Song newSong) {
		String newTitle = null;

		songList.add(newSong);
		newTitle = newSong.getTitle();
		mainClass.setDirty(true);
		updateData();
		//repaint();
		
		if (newTitle != null) {
			songJList.setSelectedValue(newTitle, true);
		}
	} //}}}
	
	//{{{ editSong()
	public void editSong() {
		int index = getSelectedSong();
		org.landis.jlyrics.song.Song newSong = null;
		javax.swing.JFrame owner = getFrame();
		org.landis.jlyrics.SongEditor songEdit = new SongEditor(owner, (Song)songList.get(index));
		songEdit.pack();
		songEdit.setLocationRelativeTo(owner);
		
		if(songEdit.saveSong() == songEdit.APPROVE_OPTION)
			newSong = songEdit.getSong();
		
		if(newSong != null) {
			songList.set(index, newSong);
			mainClass.setDirty(true);
			updateData();
			repaint();
		}
		songJList.setSelectedIndex(index);
	} //}}}
	
	//{{{ updateData()
	public void updateData() {
		Collections.sort(songList);
		songIndex.clear();
		songIndex = songList.getTitles();
		artistIndex.clear();
		artistIndex = songList.getArtists();
		resetData();
	} //}}}
	
	public void resetData() {
		songJList.setListData(songIndex);
	}
	
	//{{{ getFrame()
	public MainWindow getFrame() {
		return this;
	}//}}}
	
	//{{{ copySong()
	public void copySong() {
		((Song) songList.get(songJList.getSelectedIndex())).copy();
	} //}}}
	
	//{{{ getSelectedSong()
	public int getSelectedSong() {
		return songJList.getSelectedIndex();
	} //}}}
	
	//{{{ setSelectedSong()
	public void setSelectedSong(int i) {
		songJList.setSelectedIndex(i);
	}//}}}
	
	//{{{ makeData()
	private void makeData() {
		FileIO fileIn = new FileIO();
		songList = fileIn.xmlRead(new File("./songs.xml"));
	} //}}}
	
	//{{{ saveData()
	public void saveData() {
		FileIO xmlWrite = new FileIO();
		xmlWrite.xmlWrite(new File("./songs.xml"), songList, true);
		mainClass.setDirty(false);
	}//}}}
	
	public void ExportXML(File out) {
		FileIO xmlWrite = new FileIO();
		xmlWrite.xmlWrite(out, songList, false);
	}
	
	//}}}
	
	//{{{ Action classes
	
	//{{{ AddAction
	public class AddAction extends AbstractAction {
		public AddAction (ImageIcon icon, String text, String desc) {
			super(text, icon);
			putValue(SHORT_DESCRIPTION, desc);
		}
		
		public void actionPerformed(ActionEvent e) {
			Song newSong = null;
			MainWindow owner = getFrame();
			SongEditor songEdit = new SongEditor(owner);
			songEdit.pack();
			songEdit.setLocationRelativeTo(owner);
			
			if(songEdit.saveSong() == songEdit.APPROVE_OPTION)
				newSong = songEdit.getSong();
			
			if(newSong != null) {
				owner.addSong(newSong);
			}
		}
	}//}}}
	
	//{{{ EditAction
	public class EditAction extends AbstractAction {
		public EditAction (ImageIcon icon, String text, String desc) {
			super(text, icon);
			putValue(SHORT_DESCRIPTION, desc);
		}
		public void actionPerformed(ActionEvent e) {
			if (!songJList.isSelectionEmpty()) {
				editSong();
			} else {
				JOptionPane.showMessageDialog(getFrame(), "No Song Selected.", "Error", JOptionPane.WARNING_MESSAGE);
			}
		}
	}//}}}
	
	//{{{ DeleteAction
	public class DeleteAction extends AbstractAction {
		public DeleteAction(ImageIcon icon, String text, String desc) {
			super(text, icon);
			putValue(SHORT_DESCRIPTION, desc);
		}
		public void actionPerformed(ActionEvent e) {
			if ( !songJList.isSelectionEmpty() ) {
				songList.remove(songJList.getSelectedIndex());
				updateData();
				mainClass.setDirty(true);
			}
		}
	} //}}}
	
	//{{{ CopyAction
	public class CopyAction extends AbstractAction {
		public CopyAction (ImageIcon icon, String text, String desc) {
			super(text, icon);
			putValue(SHORT_DESCRIPTION, desc);
		}
		public void actionPerformed(ActionEvent e) {
			if (!songJList.isSelectionEmpty())
				copySong();
			else
				JOptionPane.showMessageDialog(getFrame(), "No Song Selected.", "Error", JOptionPane.WARNING_MESSAGE);
		}
	}//}}}
	
	//{{{ ImportAction
	public class ImportAction extends AbstractAction {
		public ImportAction(ImageIcon icon, String text, String desc) {
			super(text, icon);
			putValue(SHORT_DESCRIPTION, desc);
		}
		public void actionPerformed(ActionEvent e) {
			final JFileChooser fc = new JFileChooser();
			fc.addChoosableFileFilter(new songtxtFilter());
			int returnVal = fc.showOpenDialog(getFrame());
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				File file = fc.getSelectedFile();
				FileIO io = new FileIO();
				Song newSong = io.txtRead(file);
				addSong(newSong);
			}
		}
	}//}}}
	
	//{{{ ExportAction
	public class ExportAction extends AbstractAction {
		public ExportAction(ImageIcon icon, String text, String desc) {
			super(text, icon);
			putValue(SHORT_DESCRIPTION, desc);
		}
		public void actionPerformed(ActionEvent e) {
			if (!songJList.isSelectionEmpty()) {
				final JFileChooser fc = new JFileChooser();
				fc.addChoosableFileFilter(new songtxtFilter());
				int returnVal = fc.showSaveDialog(getFrame());
				Song newSong = new Song();
				
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					File file = fc.getSelectedFile();
					//if(!file.getName().toLowerCase().endsWith(".song.txt"))
						file = new File(file.getAbsolutePath()+".song.txt");
					newSong = (Song)songList.get(songJList.getSelectedIndex());
					FileIO io = new FileIO();
					io.txtWrite(file, newSong);
				}
					
			} else {
				JOptionPane.showMessageDialog(getFrame(), "No Song Selected.", "Error", JOptionPane.WARNING_MESSAGE);
			}
		}
	}//}}}
	
	//{{{ ExportAction
	public class ExportXMLAction extends AbstractAction {
		public ExportXMLAction(ImageIcon icon, String text, String desc) {
			super(text, icon);
			putValue(SHORT_DESCRIPTION, desc);
		}
		public void actionPerformed(ActionEvent e) {
			final JFileChooser fc = new JFileChooser();
			int returnVal = fc.showSaveDialog(getFrame());
			//Song newSong = new Song();
			
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				File file = fc.getSelectedFile();
				if(!file.getName().toLowerCase().endsWith(".xml"))
					file = new File(file.getAbsolutePath()+".xml");
				ExportXML(file);
			}
		}
	}//}}}
	
	//{{{ SaveAction
	public class SaveAction extends AbstractAction {
		public SaveAction(ImageIcon icon, String text, String desc) {
			super(text, icon);
			putValue(SHORT_DESCRIPTION, desc);
		}
		public void actionPerformed(ActionEvent e) {
			saveData();
		}
	}//}}}
	
	//{{{ OptionAction
	public class OptionAction extends AbstractAction {
		public OptionAction(ImageIcon icon, String text, String desc) {
			super(text, icon);
			putValue(SHORT_DESCRIPTION, desc);
		}
		public void actionPerformed(ActionEvent e) {
			OptionEditor oe = new OptionEditor(mainClass);
			oe.pack();
			oe.setLocationRelativeTo(getFrame());
			oe.setVisible(true);
		}
	}//}}}
	
	//{{{ ExitAction
	public class ExitAction extends AbstractAction {
		public ExitAction(String text, String desc) {
			super(text);
			putValue(SHORT_DESCRIPTION, desc);
		}
		public void actionPerformed(ActionEvent e) {
			mainClass.checkForDirty();
		}
	} //}}}
	
	//}}}
}
