package org.landis.jlyrics;

//{{{ imports
import javax.swing.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.*;
import java.awt.*;
import java.awt.GraphicsEnvironment;
import org.landis.jlyrics.*; //}}}

public class OptionEditor extends JFrame {
	//{{{ variables
	private boolean saveIt;
	private UIManager.LookAndFeelInfo[] lfs;
	boolean changed;
	
	JPanel pane = new JPanel();
	JComboBox LAFBox;
	JCheckBox LAFDef;
	private jLyrics mainClass;//}}}
	
	
	//{{{ OptionEditor()
	public OptionEditor(jLyrics owner) {
		super(/*owner,*/ "DS Preferences"/*, true*/);
		mainClass = owner;
		initComponents();
		setContentPane(pane);
		changed = false;
		saveIt = false;
	} //}}}
	
	//{{{ initComponents()
	public void initComponents() {
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent evt) {
				closeDialog();
			}
		});
		
		pane.setLayout(new BoxLayout(pane, BoxLayout.Y_AXIS));
		JTabbedPane tabbedPane = new JTabbedPane();
		//JPanel generalPane = new JPanel();
		//generalPane.setLayout(new BoxLayout(generalPane, BoxLayout.Y_AXIS));
		JPanel appearancePane = new JPanel();
			appearancePane.setLayout(new BoxLayout(appearancePane, BoxLayout.Y_AXIS));
		
		//{{{ Look and Feel settings
		lfs = UIManager.getInstalledLookAndFeels();
		String[] names = new String[ lfs.length ];
		String lf = UIManager.getLookAndFeel().getClass().getName();
		int index = 0;
		for(int i=0; i < names.length; i++) {
			names[i] = lfs[i].getName();
			if(lf.equals(lfs[i].getClassName()))
				index = i;
		}
		LAFBox = new JComboBox(names);
		LAFBox.setSelectedIndex(index);
		LAFBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				changed = true;
			}
		});
		appearancePane.add(LAFBox);//}}}
		
		LAFDef = new JCheckBox("Use Default Look And Feel");
		LAFDef.setSelected(false);
		LAFDef.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent evt) {
				if (evt.getStateChange() == ItemEvent.SELECTED) {
					LAFBox.setEnabled(false);
				} else {
					LAFBox.setEnabled(true);
				}
				changed = true;
			}
		});
		appearancePane.add(LAFDef);
		
		//tabbedPane.addTab("General", generalPane);
		tabbedPane.addTab("Appearance", appearancePane);
		
		/** Set up the ok and cancel buttons.
		 *
		 */
		
		JPanel footerPane = new JPanel();
		footerPane.setLayout(new BoxLayout(footerPane, BoxLayout.X_AXIS));
		footerPane.add(Box.createHorizontalGlue());
		JButton button = new JButton("OK");
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				okButtonAction(evt);
			}
		});
		footerPane.add(button);
		button = new JButton("Cancel");
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				cancelButtonAction(evt);
			}
		});
		footerPane.add(button);
		button = new JButton("Apply");
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				applyButtonAction(evt);
			}
		});
		footerPane.add(button);
		
		pane.add(tabbedPane);
		pane.add(footerPane);
	} //}}}
	
	//{{{ OptionEditor methods
	
	//{{{ closeDialog()
	private void closeDialog() {
		setVisible(false);
		dispose();
	} //}}}
	
	//{{{ getLookAndFeel()
	public int getLookAndFeel() {
		if (LAFDef.isSelected())
			return -1;
		else
			return LAFBox.getSelectedIndex();
	} //}}}
	
	//{{{ saveData()
	public void saveData() {
		mainClass.switchLookAndFeel(getLookAndFeel());
	} //}}}
	
	//{{{ applyButtonAction
	private void applyButtonAction(ActionEvent evt) {
		if (changed)
			saveData();
		SwingUtilities.updateComponentTreeUI(this);
		pack();
	} //}}}
	
	//{{{ okButtonAction
	private void okButtonAction(ActionEvent evt) {
		if(changed)
			saveData();
		closeDialog();
	} //}}}
	
	//{{{ cancelButtonAction()
	private void cancelButtonAction(ActionEvent evt) {
		closeDialog();
	} //}}}
	
	//}}}
}
