package org.landis.jlyrics;

//{{{ imports
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;
import javax.swing.border.EmptyBorder;
import java.awt.*;
import java.awt.event.*;
import org.landis.jlyrics.song.*; //}}}


public class SongEditor extends JDialog {
	//{{{ Variables
	JPanel pane = new JPanel();
	JTextField title, artist;
	JTextArea lyricArea;
	
	public static int APPROVE_OPTION = 1;
	public static int CANCEL_OPTION = 0;
	private boolean updateSong; //}}}
	
	//{{{ SongEditor()
	public SongEditor(Frame owner) {
		super(owner, "Creating New Song", true);
		initComponents();
		setContentPane(pane);
		updateSong = false;
	} //}}}
	
	//{{{ SongEditor(newsong)
	public SongEditor(Frame owner, Song newSong) {
		super(owner, "Editing " + newSong.getTitle(), true);
		initComponents();
		title.setText(newSong.getTitle());
		artist.setText(newSong.getArtist());
		lyricArea.setText(newSong.getLyrics());
		
		setContentPane(pane);
		updateSong = false;
	} //}}}
	
	//{{{ initComponents()
	private void initComponents() {
		setLocation(250,250);
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent evt) {
				closeDialog(evt);
			}
		});
		
		//{{{ Text Areas
		JLabel titleLabel = new JLabel("Title:");
		JLabel artistLabel = new JLabel("Author:");
		JLabel lyricsLabel = new JLabel("Lyrics:");
		title = new JTextField(35);
		artist = new JTextField(35);
		lyricArea = new JTextArea(8, 30);
		JScrollPane lyrics = new JScrollPane(lyricArea, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED); //}}}
		
		
		//{{{ ok, cancel, apply buttons
		JPanel bottomPane = new JPanel();
		JButton okButton = new JButton("OK"); 
		okButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				okButtonAction(evt);
			}
		});
		bottomPane.add(okButton);
		JButton cancelButton = new JButton("Cancel");
		cancelButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				cancelButtonAction(evt);
			}
		}); 
		bottomPane.add(cancelButton); //}}}
		
		//{{{ toolbar init
		JToolBar toolbar = new JToolBar();
		
		JButton cutButton = new JButton(new javax.swing.text.DefaultEditorKit.CutAction());
			cutButton.setText("");
			cutButton.setIcon(new ImageIcon("./images/cut.gif"));
			cutButton.setToolTipText("Cut");
		JButton copyButton = new JButton(new javax.swing.text.DefaultEditorKit.CopyAction());
			copyButton.setText("");
			copyButton.setIcon(new ImageIcon("./images/copy.gif"));
			copyButton.setToolTipText("Copy");
		JButton pasteButton = new JButton(new javax.swing.text.DefaultEditorKit.PasteAction());
			pasteButton.setText("");
			pasteButton.setIcon(new ImageIcon("./images/paste.gif"));
			pasteButton.setToolTipText("Paste");
		toolbar.add(cutButton);
		toolbar.add(copyButton);
		toolbar.add(pasteButton); //}}}
		
		//{{{ layout
		GridBagLayout gridbag = new GridBagLayout();
		pane.setLayout(gridbag);
		gridbag.setConstraints(toolbar, setGBConstraints(0,0,2,1,GridBagConstraints.CENTER,1.0,0.0, GridBagConstraints.HORIZONTAL));
		gridbag.setConstraints(titleLabel, setGBConstraints(0,1,1,1,GridBagConstraints.EAST,0.1,0.0, GridBagConstraints.NONE));
		gridbag.setConstraints(title, setGBConstraints(1,1,1,1,GridBagConstraints.WEST,0.9,0.0, GridBagConstraints.HORIZONTAL));
		gridbag.setConstraints(artistLabel, setGBConstraints(0,2,1,1,GridBagConstraints.EAST,0.1,0.0, GridBagConstraints.NONE));
		gridbag.setConstraints(artist, setGBConstraints(1,2,1,1,GridBagConstraints.WEST,0.9,0.0, GridBagConstraints.HORIZONTAL));
		gridbag.setConstraints(lyricsLabel, setGBConstraints(0,3,1,1,GridBagConstraints.NORTHEAST,0.1,0.0, GridBagConstraints.NONE));
		gridbag.setConstraints(lyrics, setGBConstraints(1,3,1,1,GridBagConstraints.WEST,0.9,1.0, GridBagConstraints.BOTH));
		gridbag.setConstraints(bottomPane, setGBConstraints(1,4,1,1,GridBagConstraints.EAST,0.0,0.0, GridBagConstraints.NONE));
		
		pane.setLayout(gridbag);
		pane.add(toolbar);
		pane.add(titleLabel);
		pane.add(title);
		pane.add(artistLabel);
		pane.add(artist);
		pane.add(lyricsLabel);
		pane.add(lyrics);
		pane.add(bottomPane);
		pane.setBorder(new javax.swing.border.EmptyBorder(0,3,0,1));//}}}
	} //}}}
	
	//{{{ SongEditor Methods
	
	//{{{ getSong()
	public Song getSong() {
		Song newSong = new Song();
		
		newSong.setTitle(title.getText());
		newSong.setArtist(artist.getText());
		newSong.setLyrics(lyricArea.getText());
		
		return newSong;
	} //}}}
	
	//{{{ setGBConstraints()
	protected GridBagConstraints setGBConstraints(int x, int y, int w, int h,
	                             int a, double xWeight, double yWeight, int fill) {
		GridBagConstraints c = new GridBagConstraints();
		c.gridx = x;
		c.gridy = y;
		c.gridwidth = w;
		c.gridheight = h;
		c.anchor = a;
		c.weightx = xWeight;
		c.weighty = yWeight;
		c.fill = fill;
		
		return c;
	} //}}}
	
	//{{{ closeDialog()
	private void closeDialog(WindowEvent evt) {
		setVisible(false);
		dispose();
	} //}}}
	
	//{{{ saveSong()
	public int saveSong() {
		show();
		
		if(updateSong)
			return APPROVE_OPTION;
		else
			return CANCEL_OPTION;
	} //}}}
	
	//}}}
	
	//{{{ Actions
	
	//{{{ okButtonAction()
	private void okButtonAction(ActionEvent evt) {
		if (!title.getText().trim().equals("")) {
			updateSong = true;
			closeDialog(null);
		} else {
			JOptionPane.showMessageDialog(null, "Error: No title set.");
		}
	} //}}}
	
	//{{{ cancelButtonAction()
	private void cancelButtonAction(ActionEvent evt) {
		updateSong = false;
		closeDialog(null);
	} //}}}
	
	//}}}
}
