package org.landis.jlyrics.song;

import org.landis.jlyrics.song.Song;
import java.util.*;

public class SongList extends AbstractList implements List {
	private Song elementData[];
	private int size;
	
	public SongList(int initialCapacity) {
		super();
		if (initialCapacity < 0)
			throw new IllegalArgumentException("Illegal Capacity: "+ initialCapacity);
		this.elementData = new Song[initialCapacity];
	}
	
	public SongList() {
		this(10);
	}
	
	public void ensureCapacity(int minCapacity) {
		int oldCapacity = elementData.length;
		if (minCapacity > oldCapacity) {
			Object oldData[] = elementData;
			int newCapacity = (oldCapacity * 3)/2 + 1;
			if (newCapacity < minCapacity)
				newCapacity = minCapacity;
			elementData = new Song[newCapacity];
			System.arraycopy(oldData, 0, elementData, 0, size);
		}
	}
	
	public boolean add(Object o) {
		ensureCapacity(size + 1);  // Increments modCount!!
		elementData[size++] = (Song) o;
		return true;
	}
	
	public Object set(int index, Object element) {
		RangeCheck(index);
		Object oldValue = elementData[index];
		elementData[index] = (Song) element;
		return oldValue;
	}
	
	public Object remove(int index) {
		RangeCheck(index);
		
		modCount++;
		Object oldValue = elementData[index];
		
		int numMoved = size - index - 1;
		if (numMoved > 0)
			System.arraycopy(elementData, index+1, elementData, index, numMoved);
		elementData[--size] = null; // Let gc do its work
		
		return oldValue;
	}
	
	public Vector getTitles() {
		Vector vec = new Vector(size);
		for (int i = 0; i < size; i++) {
			vec.add(((Song)get(i)).getTitle());
		}
		return vec;
	}
	
	public Vector getArtists() {
		Vector vec = new Vector(size);
		for (int i = 0; i < size; i++) {
			vec.add(((Song)get(i)).getArtist());
		}
		return vec;
	}
	
	public Object get(int index) {
		RangeCheck(index);
		
		return elementData[index];
	}
	
	public int size() {
		return size;
	}
	
	private void RangeCheck(int index) {
		if (index >= size || index < 0)
			throw new IndexOutOfBoundsException("Index: "+index+", Size: "+size);
	}
}
